#include <iostream>
#include <mutex>

#include <mysql_driver.h>
#include <mysql_connection.h>
#include <cppconn/statement.h>
#include <cppconn/resultset.h>
#include <iostream>
#include <vector>

class Singleton_sql {
public:
    // Метод для получения единственного экземпляра класса
    static Singleton_sql& getInstance() {
        static Singleton_sql instance; // Инициализация статического экземпляра
        return instance;
    }

    // Удаляем операции копирования и присваивания
    Singleton_sql(const Singleton_sql&) = delete;
    Singleton_sql& operator=(const Singleton_sql&) = delete;

// Определяем структуру для хранения данных
private: 
    struct MyData {
    int id;
    std::string name;
    double value;
    };

    
    void someFunction() {
        std::cout << "Вызов метода someFunction() у Singleton." << std::endl;
    }
    
public:
// Функция для извлечения данных из базы данных
std::vector<MyData> fetchData() {
    std::vector<MyData> data;
    sqlDB sql_init_struct;
    try {
        // Создание драйвера подключения
        sql::mysql::MySQL_Driver *driver;
        sql::Connection *con;

        // Подключение к базе данных
        driver = sql::mysql::get_mysql_driver_instance();
        con = driver->connect("tcp://"+sql_init_struct.sql_host+":3306", sql_init_struct.sql_login, sql_init_struct.sql_passwd);  // Замените 'user' и 'password' на ваши данные
        
        // Выбор базы данных
        con->setSchema(sql_init_struct.sql_db);  // Замените 'database_name' на имя вашей базы данных

        // Создание запроса
        sql::Statement *stmt = con->createStatement();
        sql::ResultSet *res = stmt->executeQuery("SELECT id, name, value FROM my_table");  // Замените 'my_table' на вашу таблицу

        // Чтение данных и запись в структуру
        while (res->next()) {
            MyData record;
            record.id = res->getInt("id");
            record.name = res->getString("name");
            record.value = res->getDouble("value");
            data.push_back(record);
        }

        // Освобождение ресурсов
        delete res;
        delete stmt;
        delete con;
    } catch (sql::SQLException &e) {
        std::cerr << "SQL error: " << e.what() << std::endl;
    }

    return data;
}

private:
    // Приватный конструктор предотвращает создание экземпляров
    Singleton_sql() {
        std::cout << "Создан экземпляр Singleton." << std::endl;
    }

    ~Singleton_sql() {
        std::cout << "Уничтожен экземпляр Singleton." << std::endl;
    }
};

int sql_init() {
    // Получаем единственный экземпляр класса Singleton
    Singleton_sql& instance1 = Singleton_sql::getInstance();
    instance1.fetchData();

    // Вывод данных из таблицы MySQL
    //instance1.someFunction("SELECT id, name, value FROM my_table");

    // Проверяем, что это один и тот же экземпляр
    /*if (&instance1 == &instance2) {
        std::cout << "Оба экземпляра совпадают." << std::endl;
    } else {
        std::cout << "Экземпляры разные." << std::endl;
    }*/

    return 0;
}
